package distanceFunction;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import dataStructures.utils.ArrayS;

public class SymmetricDistanceFunction<Index, Value>
{
	protected final Value[][] distance;

	private Map<Index, Integer> map;

	private final int noElements;

	public SymmetricDistanceFunction(final Collection<Index> indexSet)
	{
		this(indexSet, null);
	}

	public SymmetricDistanceFunction(final Collection<Index> indexSet, final Value iniValue)
	{
		this.noElements = indexSet.size();
		this.distance = (Value[][]) new Object[this.noElements - 1][];
		this.map = new HashMap<Index, Integer>((int) (indexSet.size() / 0.7));

		int in = 0;
		for (final Index index : indexSet)
		{
			this.map.put(index, in++);
		}

		for (int i = this.noElements - 1; i > 0; i--)
		{
			this.distance[this.noElements - 1 - i] = (Value[]) new Object[i];
			if (iniValue != null)
			{
				Arrays.fill(this.distance[this.noElements - 1 - i], iniValue);
			}
		}
	}

	/**
	 * Returns d(i,j).
	 * 
	 * @param i index
	 * @param j index
	 * @return
	 */
	public Value get(final Index in, final Index jn)
	{
		final int i = this.map.get(in);
		final int j = this.map.get(jn);
		if (j < i)
			return this.distance[j][i - (j + 1)];
		else
			return this.distance[i][j - (i + 1)];
	}

	public Set<Index> getIndexSet()
	{
		return this.map.keySet();
	}

	public int getNoOfElements()
	{
		return this.noElements;
	}

	public void set(final Index in, final Index jn, final Value value)
	{
		final int i = this.map.get(in);
		final int j = this.map.get(jn);

		if (j < i)
		{
			this.distance[j][i - (j + 1)] = value;
		} else
		{
			this.distance[i][j - (i + 1)] = value;
		}
	}

	@Override
	public String toString()
	{
		final StringBuffer s = new StringBuffer();
		// for (int i = 0; i <= this.distance.length; i++)
		// {
		//
		// s.append(i + "\t");
		// }
		System.out.println(ArrayS.toString(this.map.keySet(), "\t"));
		s.append('\n');
		for (int i = 0; i <= this.distance.length; i++)
		{
			s.append("_______");
		}
		s.append('\n');
		for (int i = 0; i < this.distance.length; i++)
		{
			final Value[] arr = this.distance[i];
			s.append(i + "|\t");
			for (int t = 0; t < i; t++)
			{
				s.append("\t");
			}
			for (int j = 0; j < arr.length; j++)
			{
				if (j == 0)
				{
					s.append("{");
				}
				s.append(arr[j]);

				if (j < (arr.length - 1))
				{
					s.append(",");
					s.append("\t");
				} else
				{
					s.append("},");
				}
			}
			s.append('\n');
		}
		s.append('\n');
		return s.toString();
	}
}
