package benchmark.realWorldData;

import hcUtils.Utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Iterator;
import java.util.TreeSet;

import solver.approximationHClustPivot.HcPivot;
import benchmark.helpClasses.timeBoundedApplications.TimeBoundedSearchTreeAlgorithm;
import distanceFunction.SymmetricDiscreteDistanceFunction;

public class TestBench
{

	static class LogFileWriter extends OutputStreamWriter
	{
		/**
		 * Only Sys.out
		 * 
		 * @throws IOException
		 */
		public LogFileWriter()
		{
			super(System.out);
		}

		public LogFileWriter(final File file) throws FileNotFoundException
		{
			super(new FileOutputStream(file));
		}

	}

	public static void main(final String[] args) throws Exception
	{

		final int Time = 30;

		if (args.length != 5)
		{
			System.err.println("wrong number of arguments");
			return;
		}

		final String filePath = args[0];
		final String logFileName = args[1];
		final int M = Integer.valueOf(args[2]);
		final double fact = Double.valueOf(args[3]);
		final int instnumber = Integer.valueOf(args[4]);

		final boolean deb = false;

		class containsSuffix implements FilenameFilter
		{
			public String sfx;

			public containsSuffix(final String s)
			{
				this.sfx = s;
			}

			@Override
			public boolean accept(final File dir, final String name)
			{
				if (name.endsWith(this.sfx))
					return true;
				return false;
			}
		}
		;

		final File folder = new File(filePath);
		final LogFileWriter logWriter = new LogFileWriter(new File(logFileName));
		// final LogFileWriter logWriter = new LogFileWriter();

		// use TreeSet to sort files
		final TreeSet<String> S = new TreeSet<String>();
		for (final String fileName : folder.list(new containsSuffix(".cm")))
		{
			S.add(fileName);
		}

		System.out.println("Number of Instances: " + S.size());

		final Iterator<String> I = S.iterator();

		int c = 0;
		while (I.hasNext())
		{
			c++;

			if (c < instnumber)
			{
				I.next();
				continue;
			}

			final String fileName = I.next();

			logWriter.write("\n" + "c " + c + " -----------------\n" + fileName + "\n");

			if (deb)
			{
				System.out.print(fileName + "\n");
			}

			int maxCorrect = 0;

			final SymmetricDiscreteDistanceFunction dist = Utils.read_cos(filePath + "/" + fileName, M, fact);
			final int n = dist.getNoOfElements();
			logWriter.write("n:=" + n + "\n");
			final int d_approx = HcPivot.run_hclustpivot(dist, 1000);
			logWriter.write("approx: " + d_approx + " \n");
			logWriter.flush();
			double sum = 0;

			for (; maxCorrect <= d_approx; maxCorrect++)
			{
				logWriter.write("\nk:" + maxCorrect);
				// System.out.println("k:" + maxCorrect);
				logWriter.flush();

				final TimeBoundedSearchTreeAlgorithm solver = new TimeBoundedSearchTreeAlgorithm(dist, maxCorrect, true, true, true, Time);

				solver.runAlgorithm();

				logWriter.flush();

				final double sec = solver.getTime();
				sum = sum + sec;

				// sumSec += sec;
				logWriter.write(" \t Time:" + sec + "s");
				logWriter.flush();

				if (!solver.isStopped())
				{
					// System.out.println("solver not stopped");

					// logWriter.write("\t iter: " +
					// solver.getNumberOfRecursiveSteps());
					// logWriter.write(", #fixedConflicts: " +
					// solver.fixedConflicts);
					// logWriter.write(", #almostFixedConflicts: " +
					// solver.almostFixedConflicts);
					// logWriter.write(", #others: " + solver.normalConflicts);
					// logWriter.write("\n");
					// solver.writeToFile(fileName + ".solu");
					if (solver.solveFlag)
					{
						final int ddd = solver.getSolver().getDistance(dist);
						logWriter.write("\t ultra:" + solver.getSolver().isUltrametric());
						logWriter.write("\t dist:" + ddd + "\n");
						logWriter.write("RESULT " + fileName + " " + n + " " + ddd + " " + d_approx + " " + sum + " " + M + " " + fact);
						logWriter.flush();

						break;
					}
				} else
				{
					logWriter.write("STOP solver stopped\n");
					logWriter.write("RESULT " + fileName + " " + n + " STOPPED " + d_approx + " " + sum + " " + M + " " + fact);
					break;

				}

			}
			// logWriter.write("sumSec:" + sumSec);

			logWriter.write("\n" + "ec " + c + "   ---------------------------------\n");
			logWriter.flush();

		}
		logWriter.close();
	}
}
