package dataReductionRules;

import java.util.Collection;
import java.util.Stack;

import solver.hierarchicalClustering.HierarchicalClusteringSolver;
import solver.hierarchicalClustering.ModificationStatistic;
import solver.hierarchicalClustering.SearchTreeSolver;
import solver.hierarchicalClustering.SolverInterface;
import distanceFunction.ReadOnlyViewOnSymmetricDiscreteDistanceFunction;
import distanceFunction.SymmetricDiscreteDistanceFunction;

public class ChangeLogger implements SolverInterface
{
	class DeleteDataChange implements ChangeInterface
	{
		private final boolean del;

		private final int i;

		public DeleteDataChange(final int i, final boolean deletedData)
		{
			this.i = i;
			this.del = deletedData;
		}

		@Override
		public void reverseChange(final HierarchicalClusteringSolver solver)
		{
			solver.setDeleteStatus(this.i, this.del);
		}

	}

	public static class DistanceChange implements ChangeInterface
	{
		private final boolean b;

		private final int c;

		private final int i;

		private final int j;

		public DistanceChange(final int i, final int j, final int c, final boolean decreseParam)
		{
			this.i = i;
			this.j = j;
			this.c = c;
			this.b = decreseParam;
		}

		public int getC()
		{
			return this.c;
		}

		public int getI()
		{
			return this.i;
		}

		public int getJ()
		{
			return this.j;
		}

		@Override
		public void reverseChange(final HierarchicalClusteringSolver solver)
		{
			solver.changeDistanceValue(this.i, this.j, -this.c, !this.b);
			if (SearchTreeSolver.RulesVerboseLevel > 1)
			{
				System.out.println("reverse:" + this.i + "," + this.j + "=" + -this.c);
			}
		}

	}

	private final Stack<ChangeInterface> changeStack;

	private final ModificationStatistic mod;

	private final HierarchicalClusteringSolver solver;

	// private final int level;

	public ChangeLogger(final HierarchicalClusteringSolver solver)
	{
		this.solver = solver;
		this.changeStack = new Stack<ChangeInterface>();
		this.mod = solver.modStatistic.clone();
		// this.level = level;
	}

	public void changeDistanceValue(final int i, final int j, final int c, final boolean decreseParam)
	{
		this.changeStack.push(new DistanceChange(i, j, c, decreseParam));
		this.solver.changeDistanceValue(i, j, c, decreseParam);
	}

	public void deleteData(final int i)
	{
		this.changeStack.push(new DeleteDataChange(i, this.solver.isDeletedData(i)));
		this.solver.deleteData(i);
	}

	@Override
	public ReadOnlyViewOnSymmetricDiscreteDistanceFunction getConflictEdgeStatistic()
	{
		return this.solver.getConflictStorage().getConflictEdgeStatistic();
	}

	/**
	 * Returns a unmodifiable collection of all conflicts.
	 * 
	 * @return
	 */
	@Override
	public Collection<int[]> getConflicts()
	{
		return this.solver.getConflictStorage().getConflicts();
	}

	// public int getCurrentSearchTreeLevel()
	// {
	// return level;
	// }

	@Override
	public SymmetricDiscreteDistanceFunction getDistanceFunction()
	{
		return this.solver;
	}

	public ModificationStatistic getModStatistic()
	{
		return this.solver.modStatistic;
	}

	@Override
	public ReadOnlyViewOnSymmetricDiscreteDistanceFunction getNonConflictEdgeStatistic()
	{
		return this.solver.getConflictStorage().getNonConflictEdgeStatistic();
	}

	public int getNumberOfConflicts()
	{
		return this.solver.getConflictStorage().getConflicts().size();
	}

	@Override
	public int getParameterValue()
	{
		return this.solver.getParameterValue();
	}

	public void reverseAllChanges()
	{
		this.solver.modStatistic = this.mod;
		while (!this.changeStack.empty())
		{
			this.changeStack.pop().reverseChange(this.solver);
		}
	}
}