package dataReductionRules;

import java.util.Collection;
import java.util.Iterator;
import java.util.PriorityQueue;
import java.util.Vector;

import org.apache.commons.lang3.mutable.MutableBoolean;
import org.apache.commons.lang3.mutable.MutableInt;

import solver.hierarchicalClustering.Conflicts;
import solver.hierarchicalClustering.SearchTreeSolver;
import dataStructures.ComparableStore;
import dataStructures.Store;
import distanceFunction.ReadOnlyViewOnSymmetricDiscreteDistanceFunction;
import distanceFunction.SymmetricBooleanArrayDistanceFunction;

/**
 * 
 * (k+1)-rule
 * 
 * @author Sepp Hartung
 * @version 1.0
 * 
 */
public class Rule1 extends AdvancedRule implements Cloneable
{
	public static class EdgeContainer2 implements EdgeContainerInterface
	{
		final SymmetricBooleanArrayDistanceFunction arr2;

		public EdgeContainer2(final int n)
		{
			this.arr2 = new SymmetricBooleanArrayDistanceFunction(n);
		}

		@Override
		public void addConflict(final int[] value)
		{
			this.addEdge(value[0], value[1]);
			this.addEdge(value[0], value[2]);
			this.addEdge(value[1], value[2]);
		}

		@Override
		public void addEdge(final int i, final int l)
		{
			this.arr2.set(i, l, true);
		}

		@Override
		public boolean containsEdge(final int i, final int l)
		{
			return this.arr2.get(i, l);
		}
	}

	public static final String EDGE_STATISTIC_KEYWORD = "edge_counter";

	public static final MutableInt maxUnsuccess = new MutableInt(500);

	public static final MutableInt queueSize = new MutableInt(7500);

	public static final String ruleKey = "k+1 rule";

	public static final MutableBoolean useSumAsCount = new MutableBoolean(false);

	public static boolean areConflictEdgesDisjointToKnown(final EdgeContainerInterface edges, final int[] conflict)
	{
		return !edges.containsEdge(conflict[0], conflict[1]) && !edges.containsEdge(conflict[1], conflict[2])
				&& !edges.containsEdge(conflict[0], conflict[2]);
	}

	private static int countConflictsContaining_i_j(final ReadOnlyViewOnSymmetricDiscreteDistanceFunction dist, final Collection<int[]> conflicts,
			final int i, final int j, final ReadOnlyViewOnSymmetricDiscreteDistanceFunction conflictEdge, int possibleIncreaseOnLowerBound,
			final EdgeContainerInterface edges)
	{
		if (conflicts.size() < (conflictEdge.getNoOfElements() * 2))
		{
			for (final int[] conflict : conflicts)
			{
				int l;
				if ((l = Conflicts.containsEdge(conflict, i, j)) >= 0)
				{// third edge
					edges.addEdge(i, l);
					edges.addEdge(j, l);

					possibleIncreaseOnLowerBound--;
				}
			}
		} else
		{
			for (int l = 0; l < conflictEdge.getNoOfElements(); l++)
			{
				if ((l != i) && (l != j))
				{
					dist.isConflict(i, j, l);
					edges.addEdge(i, l);
					edges.addEdge(j, l);
					possibleIncreaseOnLowerBound--;
				}
			}
		}
		return possibleIncreaseOnLowerBound;
	}

	public static int getConflictCount(final ReadOnlyViewOnSymmetricDiscreteDistanceFunction conflictEdge,
			final ReadOnlyViewOnSymmetricDiscreteDistanceFunction nonConflictEdge, final int[] con)
	{
		if (Rule1.useSumAsCount.booleanValue())
			return conflictEdge.get(con[0], con[1]) + conflictEdge.get(con[1], con[2]) + conflictEdge.get(con[0], con[2])
					+ nonConflictEdge.get(con[0], con[1]) + nonConflictEdge.get(con[1], con[2]) + nonConflictEdge.get(con[0], con[2]);
		else
		{
			final int con_0_1 = conflictEdge.get(con[0], con[1]);
			// int nonCon_0_1 = nonConflictEdge.get(con[0], con[1]);
			// int con_0_2 = conflictEdge.get(con[0], con[2]);
			final int nonCon_0_2 = nonConflictEdge.get(con[0], con[2]);
			// int con_1_2 = conflictEdge.get(con[1], con[2]);
			final int nonCon_1_2 = nonConflictEdge.get(con[1], con[2]);
			// return Math.max(Math.max(con_0_1, nonCon_0_1),
			// Math.max(Math.max(con_0_2, nonCon_0_2), Math.max(con_1_2,
			// nonCon_1_2)));
			return Math.max(Math.max(con_0_1, nonCon_0_2), nonCon_1_2);
		}
	}

	/**
	 * Returns a lower bound on the modifications that have to be done to solve all conflicts where the edge between i and j does not occur.
	 * 
	 * @param conflicts
	 * @param i
	 * @param j
	 * @param requiredNumberOfMod
	 * @param nonConflictEdge
	 * @param conflictEdge
	 * @param conflictsOfIandJ
	 * @return
	 */
	private static int getLowerBound(final ReadOnlyViewOnSymmetricDiscreteDistanceFunction dist, final int n, final Collection<int[]> conflicts,
			final int i, final int j, final int requiredNumberOfMod, final ReadOnlyViewOnSymmetricDiscreteDistanceFunction conflictEdge,
			final ReadOnlyViewOnSymmetricDiscreteDistanceFunction nonConflictEdge)
	{
		int lowerBound = 0;
		int possibleIncreaseOnLowerBound = conflicts.size();

		if (((lowerBound + possibleIncreaseOnLowerBound) < requiredNumberOfMod) || (lowerBound > requiredNumberOfMod))
			return lowerBound;

		final EdgeContainerInterface edges = new EdgeContainer2(n);
		edges.addEdge(i, j);

		possibleIncreaseOnLowerBound = Rule1.countConflictsContaining_i_j(dist, conflicts, i, j, conflictEdge, possibleIncreaseOnLowerBound, edges);

		if (((lowerBound + possibleIncreaseOnLowerBound) < requiredNumberOfMod) || (lowerBound > requiredNumberOfMod))
			return lowerBound;

		final Store<Iterator<int[]>, PriorityQueue<ConflictVal>> s = Rule1.iniConflictQueue(conflicts, conflictEdge, nonConflictEdge, edges,
				Rule1.queueSize.intValue());
		final PriorityQueue<ConflictVal> conflictSet = s.getS2();
		final Iterator<int[]> iterator = s.getS1();

		while (!conflictSet.isEmpty())
		{
			final int[] conflict = conflictSet.poll().getValue();

			if (Rule1.areConflictEdgesDisjointToKnown(edges, conflict))
			{
				lowerBound++;

				edges.addEdge(conflict[0], conflict[1]);
				edges.addEdge(conflict[0], conflict[2]);
				edges.addEdge(conflict[1], conflict[2]);
			} else
			{
				possibleIncreaseOnLowerBound--;
			}

			if (((lowerBound + possibleIncreaseOnLowerBound) < requiredNumberOfMod) || (lowerBound > requiredNumberOfMod))
				return lowerBound;

			while (iterator.hasNext())
			{
				final int[] conTmp = iterator.next();
				if (Rule1.areConflictEdgesDisjointToKnown(edges, conTmp))
				{
					conflictSet.add(new ConflictVal(Rule1.getConflictCount(conflictEdge, nonConflictEdge, conTmp), conTmp));
					break;
				}
			}
		}
		return lowerBound;
	}

	/**
	 * Arranges all conflicts (disjoint to a given set of edges) up to a given max number in a queue such that polling this queue provides conflicts
	 * with increasing (heuristically) probability of intersecting other conflicts.
	 * 
	 * @param conflicts
	 * @param conflictEdge
	 * @param nonConflictEdge
	 * @param edges
	 * @return
	 */
	public static Store<Iterator<int[]>, PriorityQueue<ConflictVal>> iniConflictQueue(final Collection<int[]> conflicts,
			final ReadOnlyViewOnSymmetricDiscreteDistanceFunction conflictEdge,
			final ReadOnlyViewOnSymmetricDiscreteDistanceFunction nonConflictEdge, final EdgeContainerInterface edges, final int maxQueueSize)
	{
		final Iterator<int[]> iterator = conflicts.iterator();
		final PriorityQueue<ConflictVal> conflictSet;
		{// ini queue
			final Vector<ConflictVal> tmpVec = new Vector<>(Math.max(1, Math.min(maxQueueSize, conflicts.size())));
			int count = 0;
			while ((count <= maxQueueSize) && iterator.hasNext())
			{
				final int[] con = iterator.next();
				if (Rule1.areConflictEdgesDisjointToKnown(edges, con))
				{
					tmpVec.add(new ConflictVal(Rule1.getConflictCount(conflictEdge, nonConflictEdge, con), con));
					count++;
					if (count >= maxQueueSize)
					{
						break;
					}
				}
			}
			conflictSet = new PriorityQueue<ConflictVal>(tmpVec);
		}
		return new Store<Iterator<int[]>, PriorityQueue<ConflictVal>>(iterator, conflictSet);
	}

	private static boolean reduce(final ChangeLogger solver, final ReadOnlyViewOnSymmetricDiscreteDistanceFunction conflictEdge,
			final ReadOnlyViewOnSymmetricDiscreteDistanceFunction nonConflictEdge, final int i, final int j, final int lowerBound)
			throws UnsolvableException
	{
		final boolean greater = (conflictEdge.get(i, j) + lowerBound) > solver.getParameterValue();
		final boolean less = (nonConflictEdge.get(i, j) + lowerBound) > solver.getParameterValue();
		if ((greater && less) || ((greater || less) && (solver.getParameterValue() == 0)))
			throw new UnsolvableException();
		else if (greater || less)
		{
			if (greater)
			{// decrease d_i_j
				solver.changeDistanceValue(i, j, -1, true);
			} else
			{// increase
				solver.changeDistanceValue(i, j, 1, true);
			}
			if (SearchTreeSolver.RulesVerboseLevel > 1)
			{
				System.out.print(Rule1.ruleKey + ":d(" + i + "," + j + ")" + (greater ? "--" : "++") + ", k=" + solver.getParameterValue());
				System.out.println("\t#conflicts:" + solver.getConflicts().size());
			}
			return true;
		}
		return false;
	}

	public Rule1(final int interleavingConst)
	{
		super(interleavingConst);
	}

	@Override
	public boolean performReductionRule(final ChangeLogger solver) throws UnsolvableException
	{
		final ReadOnlyViewOnSymmetricDiscreteDistanceFunction conflictEdge = solver.getConflictEdgeStatistic();
		final ReadOnlyViewOnSymmetricDiscreteDistanceFunction nonConflictEdge = solver.getNonConflictEdgeStatistic();

		final Vector<ComparableStore<Store<Integer, Integer>, Integer>> vec = new Vector<ComparableStore<Store<Integer, Integer>, Integer>>(
				(conflictEdge.getNoOfElements() * (conflictEdge.getNoOfElements() - 1)) / 2);

		for (int i = 0; i < conflictEdge.getNoOfElements(); i++)
		{
			for (int j = i + 1; j < conflictEdge.getNoOfElements(); j++)
			{
				final int req = solver.getParameterValue() - Math.max(conflictEdge.get(i, j), nonConflictEdge.get(i, j));
				vec.add(new ComparableStore<Store<Integer, Integer>, Integer>(new Store<Integer, Integer>(i, j), req));
			}
		}
		final PriorityQueue<ComparableStore<Store<Integer, Integer>, Integer>> queue = new PriorityQueue<>(vec);
		ComparableStore<Store<Integer, Integer>, Integer> pair;
		int unsuccessfull = 0;

		boolean out = false;
		final ReadOnlyViewOnSymmetricDiscreteDistanceFunction dist = solver.getDistanceFunction().getUnmodifiableView();
		while ((unsuccessfull <= Rule1.maxUnsuccess.intValue()) && ((pair = queue.poll()) != null))
		{
			final int i = pair.getS1().getS1();
			final int j = pair.getS1().getS2();
			final int requiredNumberOfMod = solver.getParameterValue() - Math.max(conflictEdge.get(i, j), nonConflictEdge.get(i, j));
			final int lowerBound = Rule1.getLowerBound(dist, conflictEdge.getNoOfElements(), solver.getConflicts(), i, j, requiredNumberOfMod,
					conflictEdge, nonConflictEdge);

			if (Rule1.reduce(solver, conflictEdge, nonConflictEdge, i, j, lowerBound))
			{
				out = true;
				unsuccessfull = 0;
				final int req = solver.getParameterValue() - Math.max(conflictEdge.get(i, j), nonConflictEdge.get(i, j));
				queue.add(new ComparableStore<Store<Integer, Integer>, Integer>(new Store<Integer, Integer>(i, j), req));
			} else
			{
				unsuccessfull++;
			}

		}

		// if number of conflicts > k^2 and Rule1 has been applied, the instance
		// cannot be solved.
		if (!out && queue.isEmpty() && (solver.getNumberOfConflicts() > Math.pow(solver.getParameterValue(), 2)))
			throw new UnsolvableException();
		return out;
	}
}
