package dataReductionRules;

import java.util.Vector;

import solver.hierarchicalClustering.SearchTreeSolver;

/**
 * If two edges of a conflict are already fixed, then the conflict is solved by adjusting the third edge.
 * 
 * @author sepphar
 * 
 */
public class Rule6 extends AdvancedRule
{

	public Rule6(final int interleavingConst)
	{
		super(interleavingConst);
	}

	@Override
	public boolean performReductionRule(final ChangeLogger instance) throws UnsolvableException
	{
		boolean performedChanges = false;

		final Vector<int[]> changes = new Vector<>();

		for (final int[] conflict : instance.getConflicts())
		{
			final int i = conflict[0];
			final int j = conflict[1];
			final int l = conflict[2];

			final int m_i_j = instance.getModStatistic().isDecreasePossible(i, j) ? 1 : 0;
			final int m_i_l = instance.getModStatistic().isIncreasePossible(i, l) ? 1 : 0;
			final int m_j_l = instance.getModStatistic().isIncreasePossible(j, l) ? 1 : 0;
			final int sum = m_i_j + m_j_l + m_i_l;

			if (sum == 0)
				throw new UnsolvableException();
			else if (sum == 1)
			{
				performedChanges = true;
				if (m_i_j == 1)
				{
					if (SearchTreeSolver.RulesVerboseLevel > 1)
					{
						System.out.println("rule:\t" + i + "\t" + j + "\t-1");
					}
					// changes.add(new DistanceChange(i, j, -1, true));
					changes.add(new int[] { i, j, -1, l });

				} else if (m_i_l == 1)
				{
					if (SearchTreeSolver.RulesVerboseLevel > 1)
					{
						System.out.println("rule:\t" + i + "\t" + l + "\t 1");
					}
					// changes.add(new DistanceChange(i, l, 1, true));
					changes.add(new int[] { i, l, 1, j });
				} else
				{
					if (SearchTreeSolver.RulesVerboseLevel > 1)
					{
						System.out.println("rule:\t" + j + "\t" + l + "\t 1");
					}
					// changes.add(new DistanceChange(j, l, 1, true));
					changes.add(new int[] { j, l, 1, i });
				}
			}
		}

		for (final int[] change : changes)
		{
			if (instance.getDistanceFunction().isConflict(change[0], change[1], change[3]) != null)
			{
				instance.changeDistanceValue(change[0], change[1], change[2], true);
			}
		}
		return performedChanges;
	}
}
