package dataReductionRules.rule3HelpClasses;

import java.util.Vector;

/**
 * Clique in this context is a set of vertices (modeled as integers) which are equivalent according to some quality.
 * 
 * This restriction is a convention and isn't enforced in any way.
 * 
 * Furthermore since all nodes are equivalent, they can be represented by any one node from the set. This node is called representing node.
 * 
 * @author Tomasz Przedmojski
 */
public class Clique extends SparseIntegerSet implements IntegerIdentifiable
{
	private final int number;

	public Clique(final int maxElements, final int number)
	{
		super(maxElements);
		this.number = number;
	}

	public int any()
	{
		return this.getDenseArray()[0];
	}

	@Override
	public boolean equals(final Object obj)
	{
		if (obj == null)
			return false;
		if (this.getClass() != obj.getClass())
			return false;
		final Clique other = (Clique) obj;
		if (this.number != other.number)
			return false;
		return true;
	}

	@Override
	public int getNumber()
	{
		return this.number;
	}

	@Override
	public int hashCode()
	{
		final int hash = this.number % 31;
		return hash;
	}

	@Override
	public String toString()
	{
		return super.toString();
	}

	@SuppressWarnings("UseOfObsoleteCollectionType")
	public Vector<Integer> toVector()
	{
		final Vector<Integer> vector = new Vector<Integer>(this.size());
		final int[] array = this.getDenseArray();
		for (int i = 0; i < this.size(); ++i)
		{
			vector.add(array[i]);
		}
		return vector;
	}

}
