package dataReductionRules.rule3HelpClasses;

import dataReductionRules.ChangeLogger;

/**
 * 
 * @author Tomasz Przedmojski
 * 
 */
public class ThresholdGraph extends MicroGraph
{

	private final int threshold;

	/**
	 * Computes threshold graph given an instance of ChangeLogger encapsulating distance function.
	 * 
	 * Distance function is treated as a complete graph with distances being edge weights.
	 * 
	 * Threshold graph is a copy of this graph with edges of weight > threshold removed.
	 * 
	 * @param source ChangeLogger instance encapsulating distance function
	 * @param threshold cut-off threshold for edge weights in resulting graph
	 */
	public ThresholdGraph(final ChangeLogger source, final int threshold)
	{
		super(source.getDistanceFunction().getNoOfElements());
		this.threshold = threshold;

		/*
		 * since distance function is equivalent to complete symmetric graph, iterate over all distinct pairs of vertices
		 */
		for (int u = 0; u < this.maxElement; ++u)
		{
			for (int v = u + 1; v < this.maxElement; ++v)
			{
				if (source.getDistanceFunction().get(u, v) <= this.threshold)
				{
					this.addEdge(u, v);

				}
			}
			this.addEdge(u, u);
		}
	}

	public ThresholdGraph(final int maxElements)
	{
		super(maxElements);
		this.threshold = maxElements;
		for (int u = 0; u < maxElements; ++u)
		{
			this.addEdge(u, u);
		}
	}

	/**
	 * Returns value of the threshold for this graph.
	 * 
	 * @return threshold
	 */
	public int getThreshold()
	{
		return this.threshold;
	}

}
