package solver.approximationHClustPivot;

import java.util.Random;
import java.util.Vector;

import dataReductionRules.deprecated.Rule3;
import distanceFunction.SymmetricDiscreteDistanceFunction;

public class HClustPivotAndDataReduction
{

	int M;

	Random r;

	SymmetricDiscreteDistanceFunction z;

	public HClustPivotAndDataReduction(final SymmetricDiscreteDistanceFunction D)
	{
		this.z = D;
		this.r = new Random();
		this.M = this.z.getMaxValue() - 1;
		// System.out.println("M="+M);
	}

	public void solve()
	{
		final Vector<Integer> X = new Vector<Integer>();

		for (int i = 0; i < this.z.getNoOfElements(); ++i)
		{
			X.add(i);
		}

		this.solve(X);
	}

	public void solve(final Vector<Integer> X)
	{
		if (X.size() == 0)
			return;

		// interleaving with data reduction Rule 3
		final Rule3 rule = new Rule3();
		try
		{
			// int i=0;
			while (rule.performRuleSubinst(this.z, X))
			{
				;// {++i;System.out.print(i+" ");}
			}
		} catch (final Exception e)
		{
			System.out.println("Error in function HClustPivotAndDataReduction.solve(X)");
		}
		;

		// final FindOptimalSolutionWithinTime Inst = new
		// FindOptimalSolutionWithinTime(1, this.z,
		// 20000);
		// Inst.runAlgorithm();

		if (X.size() == 0)
			return;

		final int k = X.get(this.r.nextInt(X.size()));
		// System.out.println("pivot:" +k);

		final Vector<Vector<Integer>> P = new Vector<Vector<Integer>>();
		for (int t = 1; t <= (this.M + 1); ++t)
		{
			P.add(new Vector<Integer>());
		}
		for (int l = 0; l < X.size(); ++l)
		{
			for (int t = 1; t <= (this.M + 1); ++t)
			{
				final int i = X.get(l);
				if ((i != k) && (this.z.get(k, i) == t))
				{
					P.get(t - 1).add(i);
				}

			}
		}
		P.get(0).add(k);

		for (int t1 = 1; t1 <= (this.M + 1); ++t1)
		{
			for (int t2 = t1 + 1; t2 <= (this.M + 1); ++t2)
			{
				// System.out.println(t1+" "+t2 );
				final Vector<Integer> X1 = P.get(t1 - 1);
				final Vector<Integer> X2 = P.get(t2 - 1);
				for (int i = 0; i < X1.size(); ++i)
				{
					for (int j = 0; j < X2.size(); ++j)
					{

						this.z.set(X1.get(i), X2.get(j), t2);
					}
				}

			}
		}
		for (int t = 1; t <= (this.M + 1); ++t)
		{
			final Vector<Integer> Xt = P.get(t - 1);
			for (int i = 0; i < Xt.size(); ++i)
			{
				for (int j = i + 1; j < Xt.size(); ++j)
				{
					final int d = this.z.get(Xt.get(i), Xt.get(j));
					this.z.set(Xt.get(i), Xt.get(j), d < t ? d : t);
				}
			}
		}

		for (int t = 2; t <= (this.M + 1); ++t)
		{
			this.solve(P.get(t - 1));
		}
	}

}
