package solver.approximationHClustPivot.hcUtils;

import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.graph.SimpleDirectedGraph;
import org.jgrapht.graph.SimpleGraph;

import dataReductionRules.rule3HelpClasses.deprecated.Node;
import dataReductionRules.rule3HelpClasses.deprecated.findCCGraph;
import distanceFunction.SymmetricDiscreteDistanceFunction;

public class BuildTree
{

	public int index;

	public BuildTree()
	{
		this.index = 0;
	}

	public SimpleDirectedGraph<TNode, DefaultEdge> buildTree(final SymmetricDiscreteDistanceFunction distanceFunction)
	{
		final Vector<Integer> ind = new Vector<Integer>();
		for (int i = 0; i < distanceFunction.getNoOfElements(); ++i)
		{
			ind.add(i);
		}
		return this.buildTree(distanceFunction, ind);
	}

	public SimpleDirectedGraph<TNode, DefaultEdge> buildTree(final SymmetricDiscreteDistanceFunction distanceFunction, final Vector<Integer> indices)
	{

		// create root vertex
		final SimpleDirectedGraph<TNode, DefaultEdge> T = new SimpleDirectedGraph<TNode, DefaultEdge>(DefaultEdge.class);
		final TNode root = new TNode(this.index);
		++this.index;
		T.addVertex(root);

		// determine Max
		int Max = 0;
		for (int j = 0; j < indices.size(); j++)
		{
			for (int t = j + 1; t < indices.size(); t++)
			{
				Max = distanceFunction.get(indices.get(j), indices.get(t)) > Max ? distanceFunction.get(indices.get(j), indices.get(t)) : Max;
			}
		}

		// start recursion
		this.buildTreeRec(distanceFunction, indices, root, Max, T);

		return T;
	}

	public void buildTreeRec(final SymmetricDiscreteDistanceFunction distanceFunction, final Vector<Integer> indices, final TNode pap, final int Max,
			final SimpleDirectedGraph<TNode, DefaultEdge> T)
	{
		// Recursion base
		if (Max == 1)
		{
			for (int i = 0; i < indices.size(); ++i)
			{
				final TNode c = new TNode(this.index, indices.get(i));
				T.addVertex(c);
				this.index++;
				T.addEdge(pap, c);
			}

			return;
		}

		// build treshold graph
		// - create vertices
		final SimpleGraph<Node, DefaultEdge> G = new SimpleGraph<Node, DefaultEdge>(DefaultEdge.class);
		for (int j = 0; j < indices.size(); j++)
		{
			G.addVertex(new Node(indices.get(j), G));
		}
		// - determine edges
		final Iterator<Node> I = G.vertexSet().iterator();
		while (I.hasNext())
		{
			final Node v = I.next();
			final Iterator<Node> J = G.vertexSet().iterator();
			while (J.hasNext())
			{
				final Node w = J.next();
				if ((v != w) && (distanceFunction.get(v.i, w.i) < Max))
				{
					G.addEdge(v, w);
				}
			}
		}

		// VisTools.show(G);

		// Construct critical clique graph
		// every isolated clique of cc graph is a Max-cluster
		final SimpleGraph<TreeSet<Node>, DefaultEdge> CC = findCCGraph.ccgraph(G);
		// VisTools.show(CC);

		// Recursion
		final Iterator<TreeSet<Node>> VCC = CC.vertexSet().iterator();
		while (VCC.hasNext())
		{
			final TreeSet<Node> v = VCC.next();
			// add new tree node
			final TNode c = new TNode(this.index);
			T.addVertex(c);
			this.index++;
			// make it adjacent with its parent
			T.addEdge(pap, c);
			// copy items coerresponding to the vertices in the critical clique
			// into indc
			final Vector<Integer> indc = new Vector<Integer>();
			final Iterator<Node> vi = v.iterator();
			while (vi.hasNext())
			{
				indc.add(vi.next().i);
			}

			// recurion
			this.buildTreeRec(distanceFunction, indc, c, Max - 1, T);

		}

	}

}
