package solver.approximationHClustPivot.hcUtils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.graph.SimpleGraph;

import dataReductionRules.rule3HelpClasses.deprecated.Node;
import distanceFunction.SymmetricDiscreteDistanceFunction;

public class Utils
{

	/*
	 * Computes the L1-norm between two distance functions D1 and D2 - Precondition: D1 and D2 have same size
	 */
	public static int distance(final SymmetricDiscreteDistanceFunction D1, final SymmetricDiscreteDistanceFunction D2)
	{
		int d = 0;
		for (int i = 0; i < D1.getNoOfElements(); ++i)
		{
			for (int j = i + 1; j < D1.getNoOfElements(); ++j)
			{
				d = d + Math.abs(D1.get(i, j) - D2.get(i, j));
			}
		}

		return d;
	}

	public static SymmetricDiscreteDistanceFunction rd(final TreeSet<StrDouble> S, final int M, final double fact, final int dim,
			final TreeMap<String, Integer> EI)
	{
		final Iterator<StrDouble> I = S.iterator(); // iterator, ascending order
		final Vector<StrPair> V = new Vector<StrPair>();
		while (I.hasNext())
		{
			final StrDouble p = I.next();
			// System.out.println(p.p.s1+" - "+p.p.s2+" -- "+p.d);
			V.add(p.p);
		}

		final int n = V.size(); // number of pairs
		final int t = M;

		final SymmetricDiscreteDistanceFunction dist = new SymmetricDiscreteDistanceFunction(dim);
		final TreeMap<Integer, Integer> Count = new TreeMap<Integer, Integer>();

		for (int i = 0; i < n; ++i)
		{
			final String s1 = V.get(i).s1;
			final String s2 = V.get(i).s2;
			final int i1 = EI.get(s1);
			final int i2 = EI.get(s2);

			// compute new distance
			int d = 0;
			d = 1;
			for (int j = 1; j <= t; ++j)
			{
				final int tr = (int) (n * Math.pow(fact, j));
				if (i >= tr)
				{
					++d;
				}
			}

			dist.set(i1, i2, d);
			if (Count.get(d) == null)
			{
				Count.put(d, 1);
			} else
			{
				Count.put(d, Count.get(d) + 1);
			}
		}

		try
		{
			dist.writeToFile("/scratch/uhlmann_j/ddata.m", null);
		} catch (final Exception e)
		{
		}

		// System.out.println("number of pairs "+n);
		for (int i = 0; i < (2 * (t + 10)); ++i)
		{
			final Integer cc = Count.get(i);
			if (cc != null)
			{
				;// System.out.println("d="+i+" number:"+cc);
			}
		}
		return dist;
	}

	public static SymmetricDiscreteDistanceFunction read_cos(final String file, final int Mvalue, final double fact)
	{
		final TreeSet<String> X = new TreeSet<String>(); // elements

		// all pairs of elements, sorted by distance
		final TreeSet<StrDouble> M = new TreeSet<StrDouble>();

		// Mapping from elements to Integers and vice versa
		final TreeMap<String, Integer> SI = new TreeMap<String, Integer>();
		final TreeMap<Integer, String> IS = new TreeMap<Integer, String>();

		// Distances
		final Vector<Vector<Double>> D = new Vector<Vector<Double>>();

		int n = 0; // number of elements
		double maxd = Double.MIN_VALUE;

		int l = 0;// line number
		try
		{
			// Open the file that is the first
			// command line parameter
			final FileInputStream fstream = new FileInputStream(file);
			// Get the object of DataInputStream
			final DataInputStream in = new DataInputStream(fstream);
			final BufferedReader br = new BufferedReader(new InputStreamReader(in));

			String strLine;
			int i = 0; // row of Matrix

			// Read File Line By Line
			while ((strLine = br.readLine()) != null)
			{
				++l;
				if (l == 1) // read number of elements
				{
					n = Integer.valueOf(strLine);
					// System.out.println("line "+l+": "+n);
				} else if (l <= (n + 1)) // read elements
				{
					X.add(strLine);
					SI.put(strLine, l - 2);
					IS.put(l - 2, strLine);
					// System.out.println("line " +l +":\t element: "+strLine);
				} else
				// read matrix
				{
					// String sc= new String(""+0x09);
					final StringTokenizer st = new StringTokenizer(strLine);
					// System.out.println("numb of tokens: "+st.countTokens());
					int j = i + 1;
					final Vector<Double> row = new Vector<Double>();
					while (st.hasMoreTokens())
					{
						final String s = st.nextToken();
						final double dij = Double.valueOf(s);
						row.add(dij);
						maxd = dij > maxd ? dij : maxd;
						try
						{
							final String s1 = IS.get(i);
							final String s2 = IS.get(j);
							// System.out.println(s1+ " "+s2+
							// " "+i+" "+j+" dij="+dij);
							M.add(new StrDouble(IS.get(i), IS.get(j), -dij));
						} catch (final Exception e)
						{
							System.err.println("i=" + i + "  j=" + j + "     M.add() throws exceptio: " + e.getMessage());
						}
						++j;
					}
					D.add(row);
					++i;
				}

			}
		} catch (final Exception e)
		{// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}

		// System.out.println("Max dist "+maxd);
		// output D
		// for(int i=0;i<D.size();++i)
		// {
		// for(int j=0;j <D.get(i).size();++j)
		// {
		// System.out.print(D.get(i).get(j)+"\t");
		// }
		// System.out.println("");
		// }

		return Utils.rd(M, Mvalue, fact, n, SI);
	}

	public static SymmetricDiscreteDistanceFunction readrd(final String file, final int M, final double fact)
	{
		final TreeSet<String> X = new TreeSet<String>(); // Elements
		final TreeSet<StrDouble> S = new TreeSet<StrDouble>(); // pairwise
																// distance

		try
		{
			// Open the file that is the first
			// command line parameter
			final FileInputStream fstream = new FileInputStream(file);
			// Get the object of DataInputStream
			final DataInputStream in = new DataInputStream(fstream);
			final BufferedReader br = new BufferedReader(new InputStreamReader(in));

			String strLine;
			String s1 = null;
			String s2 = null;
			String s = null;
			// Read File Line By Line
			while ((strLine = br.readLine()) != null)
			{

				// split line at tab
				final char cc = 0x09;
				final String sc = new String("" + cc);
				final StringTokenizer st = new StringTokenizer(strLine, sc);
				if (st.countTokens() != 3)
				{
					System.out.println(file + " wrong format");
				}

				int c = 1;
				while (st.hasMoreTokens())
				{
					s = st.nextToken();
					if (c == 1) // the first element
					{
						X.add(s);
						s1 = s;
					}
					if (c == 2) // the second element
					{
						X.add(s);
						s2 = s;
					}
					if (c == 3) // the distance
					{
						final double d = Double.valueOf(s);
						S.add(new StrDouble(s1, s2, d));
					}
					++c;
				}
			}
			// Close the input stream
			in.close();
		} catch (final Exception e)
		{// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}

		// Mapping form the Elements to integers 0,..,|E|-1
		final TreeMap<String, Integer> EI = new TreeMap<String, Integer>();
		final Iterator<String> EII = X.iterator();
		int c = 0;

		while (EII.hasNext())
		{
			EI.put(EII.next(), c);
			++c;
		}

		return Utils.rd(S, M, fact, X.size(), EI);
	}

	public static void show(final SimpleGraph<Node, DefaultEdge> G) throws Exception
	{
		Utils.write_dimacs(G, "/scratch/uhlmann_j/tmp_graph");
		Runtime.getRuntime().exec("/scratch/uhlmann_j/show  /scratch/uhlmann_j/tmp_graph");

	}

	public static void write_dimacs(final SimpleGraph<Node, DefaultEdge> G, final String file) throws Exception
	{
		final FileWriter writer = new FileWriter(file);
		writer.write("p edge " + G.vertexSet().size() + "  " + G.edgeSet().size() + "\n");
		final Iterator<Node> V = G.vertexSet().iterator();
		while (V.hasNext())
		{
			writer.write("n " + V.next() + "  1.0\n");
		}
		final Iterator<DefaultEdge> E = G.edgeSet().iterator();
		while (E.hasNext())
		{
			final DefaultEdge e = E.next();
			writer.write("e " + G.getEdgeSource(e) + "  " + G.getEdgeTarget(e) + "\n");
		}
		writer.close();
	}

}
