package solver.hierarchicalClustering;

import distanceFunction.SymmetricBooleanArrayDistanceFunction;

public class ModificationStatistic
{
	private final SymmetricBooleanArrayDistanceFunction decreaseImpossibleTable;

	private final SymmetricBooleanArrayDistanceFunction increaseImpossibleTable;

	private final HierarchicalClusteringSolver solver;

	public ModificationStatistic(final HierarchicalClusteringSolver solver)
	{
		this(solver, new SymmetricBooleanArrayDistanceFunction(solver.getNoOfElements()), new SymmetricBooleanArrayDistanceFunction(
				solver.getNoOfElements()));
	}

	private ModificationStatistic(final HierarchicalClusteringSolver solver, final SymmetricBooleanArrayDistanceFunction incImpTable,
			final SymmetricBooleanArrayDistanceFunction decImpTable)
	{
		this.solver = solver;
		this.increaseImpossibleTable = incImpTable;
		this.decreaseImpossibleTable = decImpTable;
	}

	public void avoidDecrease(final int i, final int j)
	{
		this.decreaseImpossibleTable.set(i, j, true);
	}

	public void avoidIncrease(final int i, final int j)
	{
		this.increaseImpossibleTable.set(i, j, true);
	}

	@Override
	public ModificationStatistic clone()
	{
		return new ModificationStatistic(this.solver, this.increaseImpossibleTable.clone(), this.decreaseImpossibleTable.clone());
	}

	public int getContentHash()
	{
		return this.decreaseImpossibleTable.hashCode() * this.increaseImpossibleTable.hashCode();
	}

	public boolean isDecreasePossible(final int i, final int j)
	{
		return !this.decreaseImpossibleTable.get(i, j);
	}

	/**
	 * Check whether distance d_i_j can be increased or not.
	 * 
	 * @param i
	 * @param j
	 */
	public boolean isIncreasePossible(final int i, final int j)
	{
		return !this.increaseImpossibleTable.get(i, j);
	}

	protected void removeDecreaseBlock(final int i, final int j)
	{
		this.decreaseImpossibleTable.set(i, j, false);
	}

	protected void removeIncreaseBlock(final int i, final int j)
	{
		this.increaseImpossibleTable.set(i, j, false);
	}
}